
yarip.Handler = function(overlay, doc)
{
	this.overlay = overlay ? overlay : null;
	this.element = null;
	this.timeout = null;
	this.xValue = null;
	this.outline = null;
	this.href = null;
	this.doc = doc ? doc : null;
	this.pages = null;
	this.address = null;
	this.handlers = [];

	this.start = function(doc, pages, mode)
	{
		if (doc) this.doc = doc;

		this.pages = pages;
		this.address = yarip.getAddress(this.doc, mode);
		var active = false;

		if (this.doc && this.doc.nodeName == "#document" && this.pages && this.address)
		{
			this.doc.addEventListener("mouseover", this, false);
			this.doc.addEventListener("mouseout", this, false);
			this.doc.addEventListener("mousedown", this, false);
			this.doc.addEventListener("keypress", this, false);
			active = true;
		}

		if (this.handlers.length > 0)
		{
			for (var i = 0; i < this.handlers.length; i++) this.handlers[i].start(null, this.pages, mode);
			active = true;
		}

		return active;
	};

	this.stop = function()
	{
		if (!this.doc || !this.pages) return;

		this.doc.removeEventListener("mouseover", this, false);
		this.doc.removeEventListener("mouseout", this, false);
		this.doc.removeEventListener("mousedown", this, false);
		this.doc.removeEventListener("keypress", this, false);

		if (this.handlers.length > 0) for (var i = 0; i < this.handlers.length; i++) this.handlers[i].stop();

		this.removeStatus();
		this.reset();
	};

	this.addHandler = function(handler)
	{
		this.handlers[this.handlers.length] = handler;
	};

	this.handleEvent = function(event)
	{
		if (event.target.localName.match(/frameset|frame|iframe/i) || event.target.id == "yarip-status-text") return;

		switch (event.type)
		{
			case "mouseover":
				this.element = event.target;
				this.outline = this.element.style.outline;
				if (!this.element.href) this.switchWithAnchorAncestor();
				this.href = this.element.href;
				this.element.removeAttribute("href");
				this.timeout = setTimeout(this.select, 150, this);
				event.stopPropagation();
				break;

			case "mouseout":
				this.reset();
				event.stopPropagation();
				break;

			case "mousedown":
				switch (event.button)
				{
					case 0: // left mouse button (remove and remember)
						this.remove();
						event.stopPropagation();
						break;

					case 1: // middle mouse button (remove temporary)
						this.remove(true);
						event.stopPropagation();
						break;

					default: // escape
						this.overlay.stop();
				}
				break;

			case "keypress":
				switch (event.keyCode)
				{
					case 27: // escape
						this.overlay.stop();
						event.stopPropagation();
						break;

					case 0: // character
						switch (event.charCode)
						{
							case 114: // character `r' (remove and remember)
								this.remove();
								event.stopPropagation();
								break;

							case 116: // character `t' (remove temporary)
								this.remove(true);
								event.stopPropagation();
						}
				}
		}
	};

	this.switchWithAnchorAncestor = function()
	{
		if (!this.element.localName.match(/^(abbr|acronym|b|basefont|bdo|big|br|button|cite|code|del|dfn|em|font|i|img|ins|kbd|label|map|q|samp|select|small|span|strong|sub|sup|textarea|tt|u|var)$/i)) return;

		var xValue = yarip.calculateXPath(this.element);
		this.xValue = xValue;
		if (xValue.match(/^[/]{2}/)) xValue = yarip.calculateXPath(this.element, true);

		var expr = xValue.match(/^.*?\/a\b[^/]*/i);
		if (!expr) return;

		var elements = yarip.getElementsByXPath(this.doc, expr[0]);
		if (!elements) return;

		this.element = elements.snapshotItem(0);
		var shortPath = yarip.calculateXPath(this.element, false);
		this.xValue = shortPath ? shortPath : expr;
	};

	this.select = function(handler)
	{
		if (handler.element.getAttribute("status") == "whitelisted") handler.element.style.outline = "3px solid #990000";
		else handler.element.style.outline = "3px solid #000099";

		if (!handler.xValue) handler.xValue = yarip.calculateXPath(handler.element);
		handler.setStatus(handler.xValue);
	};

	this.reset = function(removed)
	{
		if (this.timeout) clearTimeout(this.timeout);
		else this.timeout = null;

		this.xValue = null;

		if (removed || !this.element) return;

		this.element.style.outline = this.outline;
		if (this.href) this.element.href = this.href;
		this.element = null;
	};

	this.remove = function(removeTemporary)
	{
		if (!this.element) return;

		if (!this.xValue) this.xValue = yarip.calculateXPath(this.element);

		if (removeTemporary) yarip.removeByXPath(this.doc, this.pages, this.address, new yarip.XPath(this.xValue, this.overlay.deep), null, true);
		else yarip.removeByXPath(this.doc, this.pages, this.address, new yarip.XPath(this.xValue, this.overlay.deep), true);

		this.reset(true);
	};

	this.setStatus = function(text)
	{
		if (!text || !this.doc) return;

		var yaripStatusText = this.doc.getElementById("yarip-status-text");
		if (!yaripStatusText)
		{
			yaripStatusText = this.doc.createElement("div");
			yaripStatusText.setAttribute("id", "yarip-status-text");
			yaripStatusText.setAttribute("style",
				"background: white none repeat scroll 0% !important;" +
				"color: gray !important;" +
				"display: block !important;" +
				"font: normal normal normal 11px Arial,sans-serif !important;" +
				"margin: 25px 0 0 0 !important;" +
				"padding: 5px !important;" +
				"position: fixed !important;" +
				"text-align: left !important;" +
				"z-index: 10000 !important;");
			this.doc.body.insertBefore(yaripStatusText, this.doc.body.firstChild);
		}
		yaripStatusText.innerHTML = text;
	};

	this.removeStatus = function()
	{
		if (!this.doc) return;

		if (this.statusTimeout) clearTimeout(this.statusTimeout);

		var yaripStatusText = this.doc.getElementById("yarip-status-text");
		if (yaripStatusText) this.doc.body.removeChild(yaripStatusText);
	};
};
